/********************************************************************
*
*                
*
*********************************************************************
* FileName:        Main.c
* Dependencies:    See INCLUDES section below
* Processor: 	   PIC18F46K22      
* Compiler: 	   C-18       
* Company:         Microchip Technology, Inc.
*
* Software License Agreement:
*
* The software supplied herewith by Microchip Technology Incorporated
* (the "Company") for its PICmicro Microcontroller is intended and
* supplied to you, the Company's customer, for use solely and
* exclusively on Microchip PICmicro Microcontroller products. The
* software is owned by the Company and/or its supplier, and is
* protected under applicable copyright laws. All rights are reserved.
* Any use in violation of the foregoing restrictions may subject the
* user to criminal sanctions under applicable laws, as well as to
* civil liability for the breach of the terms and conditions of this
* license.
*
* THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
* WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
* TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
* PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
* IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
* CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*********************************************************************/

/*


* File Description: Main module with the main() function. It includes
*                   the application code with the main state machine
*                   controlling the application's behavior.
*
* Change History:
* Author               Date        Comment
* Chris Tucker      10/18/2010  First version of module code
*********************************************************************

//*****************************************************************************
// Include and Header files
//*****************************************************************************/

#include "Lcd.h"


#include <p18f46k22.h>
#include <GenericTypeDefs.h>

//*****************************************************************************
// PIC Configuration
//*****************************************************************************
#pragma config FOSC = INTIO67   // Internal OSC block, Port Function on RA6/7
#pragma config PLLCFG = OFF     // OSC used Directly; PLL -Disabled
#pragma config PRICLKEN = OFF   // Primary clock can be disabled by software
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor disabled
#pragma config IESO = OFF       // OSC switch mode disabled
#pragma config PWRTEN = OFF     // Power Up Timer is disabled
#pragma config BOREN = OFF      // Brown-out Reset disabled in Hardware and Soft
#pragma config BORV = 250       // VBOR set to 2.50 V nominal
#pragma config WDTEN = OFF      // Watch Dog Timer disabled. SWDTEN no effect
#pragma config WDTPS = 1        // Watch Dog Timer Postscale 1:1
#pragma config CCP2MX = PORTC1   // CCP2 input/output is multiplexed with RC1
#pragma config PBADEN = OFF     // PORTB<5:0> pins are Config as Digital I/O on Reset
#pragma config CCP3MX = PORTE0  // P3A/CCP3 input/output is multiplexed with RE0
#pragma config HFOFST = OFF     // HFINTOSC ouput and ready status are delayed by OSC stable status
#pragma config T3CMX = PORTB5   // T3CKl is on RB5
#pragma config P2BMX = PORTC0   // P2B is on RC0
#pragma config MCLRE = EXTMCLR  // MCLR pin enabled, RE3 input pin disabled
#pragma config STVREN = OFF     // Stack full/underflow will not cause Reset
#pragma config LVP = OFF        // Single-Supply ICSP disabled
#pragma config XINST = OFF      // Instruction set Extension and indexed Addressing mode disabled
#pragma config DEBUG = OFF      // Disabled
#pragma config CP0 = OFF        // Block 0 (000800 - 001FFFh) not code-protected
#pragma config CP1 = OFF        // Block 1 (002000 - 003FFFh) not code-protected
#pragma config CP2 = OFF        // Block 2 (004000 - 005FFFh) not code-protected
#pragma config CP3 = OFF        // Block 3 (006000 - 007FFFh) not code-protected
#pragma config CPB = OFF        // Boot Block (000000 - 0007FFh) not code-protected
#pragma config CPD = OFF        // Data EEPROM not code-protected
#pragma config WRT0 = OFF       // Block 0 (000800-001FFFh) not write-protected
#pragma config WRT1 = OFF       // Block 1 (002000-003FFFh) not write-protected
#pragma config WRT2 = OFF       // Block 2 (004000-005FFFh) not write-protected
#pragma config WRT3 = OFF       // Block 3 (006000-007FFFh) not write-protected
#pragma config WRTC = OFF       // Configuration registers (300000-3000FFh) not write protected
#pragma config WRTB = OFF       // Boot Block (000000-0007FFh) not write-protected
#pragma config WRTD = OFF       // Data EEPROM not write-protected
#pragma config EBTR0 = OFF      // Block 0 (000800 - 001FFFh) not protected from table reads executed in other blocks
#pragma config EBTR1 = OFF      // Block 0 (002000 - 003FFFh) not protected from table reads executed in other blocks
#pragma config EBTR2 = OFF      // Block 0 (004000 - 005FFFh) not protected from table reads executed in other blocks
#pragma config EBTR3 = OFF      // Block 0 (006000 - 007FFFh) not protected from table reads executed in other blocks
#pragma config EBTRB = OFF      // Block 0 (000000 - 0007FFh) not protected from table reads executed in other blocks



// Used to prevent buffer overflow
#define TEMP_UPD       	20         ///< Temperature update interval = 20*4 ms ~ 1s

#define TIMER0		0b11010111;       ///< Configure TMR0 for 4 mS increment

#define SYS_OSC		0b01010110; ///< PLL is disabled;4 MHz; Fosc is Internal


//*****************************************************************************
//Local Function Prototypes
//*****************************************************************************

void SysInit(void);                 ///< General system configuration and setup


/*******************************************************************/

void main(void)
{    	
  SysInit();                        // System Initialization
  LCDInit();			 // Go Initialize the LCD
  LCDClear();
do								
	{								
	LCDWriteStr("   Microchip");	
	LCDGoto(0,1);
	LCDWriteStr(" LCD TEST    ");	
	} while(1);	
}

///////////////////////////////////////////////////////////////////////////////
//*****************************************************************************
// SUPPORT FUNCTIONS
//*****************************************************************************
///////////////////////////////////////////////////////////////////////////////

/********************************************************************
* Function:         SysInit
* 
* PreCondition: 	None
*
* Side Effects: 	None                                                   
*/
/**
* @brief	        This function is called to setup all GPIOs according to
*                   their utilization by the application. It also sets the
*                   main clock frequency Fosc to 16 MHz. It finally calls all
*                   peripheral initialization routines (Comm, I2C, LCD, ...).
*
* @note        		Called by main() once at start up.
*/
/*******************************************************************/


void SysInit(void)
{
  TRISA = 0xFF;                        // All PortA pins as inputs
  TRISB = 0x01;
  TRISC = 0xBB;
  TRISD = 0x00;
  TRISE = 0xFF;

  LATA = 0x00;
  LATB = 0x00;
  LATC = 0x80;

//*****************************************************************************
// Analog Select; Analog = 1; Digital = 0;
//*****************************************************************************
  ANSELA = 0x01;
  ANSELB = 0x00;
  ANSELCbits.ANSC3 = 0;			// <-- Used In I2C. SCL
  ANSELCbits.ANSC4 = 0;			// <-- Used In I2C. SDA
  ANSELD = 0x00;
  ANSELE = 0x00;

// PLL is disabled; 4 MHz; Clock determined by FOSC in Config Word 1.
  OSCCON = SYS_OSC;                 	             // Set Internal RC to X MHz

//*****************************************************************************
// Timer and Interrupts
//*****************************************************************************

/*************** default clock on 16F1937 is 500 KHz at startup ***************/

// All weak pull-ups are disabled; Interrupt on rising edge of RB0/INT pin;
// Internal Instruction Cycle clock (FOSC/4); 
// Increment on high-to-low Transition on RA4/T0CKl pin; Prescaler is assigned
// to TMR0; Prescaller is set to: 1:256.

    T0CON = TIMER0;						// Configure TMR0 for 4 mS increment

    // Set up Interrupts for timer		// Set up timer itself
    TMR0H = CLEAR;                      	// Clear timer - always write upper byte first
    TMR0L = CLEAR;							// Clear Timer - lower bits
	INTCONbits.TMR0IF = CLEAR;				// Timer 0 Interrupt Flag
    INTCONbits.TMR0IE = ENABLE;				// Timer 0 Interrupt Enabled

    // Set up switch interrupt on INT0
    INTCON2bits.INTEDG0 = CLEAR;    	// Interrupt on falling edge of INT0 (switch pressed)
    INTCONbits.INT0IF = CLEAR;      	// Ensure flag is cleared
    INTCONbits.INT0IE = ENABLE;      		// Enable INT0 interrupt
    // NOTE: INT0 is ALWAYS a high priority interrupt

    // Set up global interrupts
	RCONbits.IPEN	= ENABLE;				// Enable priority levels on interrupts
    INTCONbits.GIEL = ENABLE;        		// Peripheral Interrupt enable
    INTCONbits.GIEH = ENABLE;        		// General Interrupt enable

    // Interrupt Priority
    INTCON2bits.TMR0IP = CLEAR;             // Timer0 is low priority interrupt
	IPR1bits.RC1IP	= CLEAR;				// EUSART1 Receive Interrupt Priority bit
	IPR1bits.TX1IP	= CLEAR;				// EUSART1 Transmit Interrupt Priority bit

//*****************************************************************************
// Initial Conditions
//*****************************************************************************

//  VOLTInit();            // Go Initialize ADC Module For Voltmeter Functionality
//  RTCInit();			 // Go Initialize the clock
  LCDInit();			 // Go Initialize the LCD
//  I2CInit();             // Go Initialize Temp Sensor Interface
//  SNDInit();             // Go Initilize the sound module
//  ButtonsInit();         // Go initilize the Buttons module for input operations

/*#ifdef SERIAL_COM		 // If Serial Com is being used do action; else skip
  SERInit();			 // Initialized Serieal Come
  SERTxSave('\r');		 // Carriage return
  SERTxSave('\n');		 // Line Feed
  SERSendNext();  		 // Go send next available character To initialize Transmission
#endif*/
}






